<?php
namespace Neuron\Person\Attribute\Type\Storage;

use Neuron\Db\Storage\Pager;
use Neuron\Person\Attribute\Type;
use Neuron\Person\Attribute\Type\Storage;
use Zend\Db\Adapter\Adapter;
use Zend\Stdlib\ArrayUtils;
use Throwable;

class Mysql extends \Neuron\Db\Storage implements Skeleton
{
    public function load($id, $mode = Storage::LOADBY_ID)
    {
        try {
            $select = $this->select()
                ->from($this->_tables['person_attribute_types']);

            switch ($mode) {
                case Storage::LOADBY_ID:
                    $select->where($this->__(['attr_type_id' => $id]));
                    break;
                case Storage::LOADBY_CODE:
                    $select->where($this->__(['attr_type_code' => $id]));
                default:
                    $select->where($this->__(['attr_type_id' => $id]));
            }

            return $this->fetchRow($select);
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function save(Type $type, $update = false)
    {
        try {
            /* mode update? */
            if ($update === true) {
                $update = $this->update()
                    ->table($this->_tables['person_attribute_types'])
                    ->set($this->__($type->pull()))
                    ->where($this->__(['attr_type_id' => $type->attr_type_id]));
					
				//echo str_replace('"','',$update->getSqlString()).'<br />'; exit;
                if ($this->execute($update)) {
                    /* update sukses, return model id */
                    return $type->attr_type_id;
                }
            } else {
                $insert = $this->insert()
                ->into($this->_tables['person_attribute_types'])
                ->autoincrement($this->_('attr_type_id'))
                ->values($this->__($type->pull()));
                if ($id = $this->execute($insert)) {
                    /* insert sukses, return model id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = Storage::LIST_ALL, $params = null)
    {
        /* base select */
        $select = $this->select()
            ->from(['pa' => $this->_tables['person_attribute_types']]);
        /* loadby page */
        if ($mode === Storage::LISTBY_PAGE) {
            /* get pager */
            $pager = Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('attr_code', 'pa.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('attr_name', 'pa.'), '%' . trim($ids) . '%')
                    ->unnest();
            }
        } else {
            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case Storage::LISTBY_CODE:
                    $select->where([$this->_('model_id', 'a.') => $ids]);
                    break;
                case Storage::LISTBY_NAME:
                    $select->where([$this->_('name', 'a.') => $ids]);
                    break;
                case Storage::LISTBY_DESCR:
                    $select->where([$this->_('description', 'a.') => $ids]);
                    break;
            }
        }

        try {
            $rows = $this->fetchAll($select);
            $result = new \stdClass();
            $result->list = $rows;
            $result->pager = $pager;
            $total = isset($rows->total) ? $rows->total : 0;
            $result->countTotal = $total;
            return $result;
        } catch (\Throwable $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
	
	public function remove($id)
    {
        try {
            $delete = $this->delete()
            ->from($this->_tables['person_attribute_types'])
            ->where($this->__(array('attr_type_id' => $id)));
			
			//echo str_replace('"','',$delete->getSqlString()).'<br />'; exit;
            
            if($this->execute($delete)){
                return $id;
            }
			
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
}
