<?php
namespace Neuron\Person;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;
use Neuron\Application\Framework\Entity as NAF;
use Zend\Validator\NotEmpty;
use Zend\Validator\Date;
use Zend\I18n\Validator\Alnum;
use Zend\I18n\Validator\Alpha;
use Zend\Validator\ValidatorChain;
use Zend\Validator\StringLength;


use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;
use Neuron\Person\Entity\Storage;
use Neuron\Person\Traits\Model;
use Zend\Filter\Digits as FilterDigits;
use Zend\I18n\Filter\Alnum as FilterAlnum;
use Zend\Validator\Digits;
use Zend\Validator\InArray;

class Entity extends \Neuron\Generic\Entity\Mapper {
    use Model;
    
    CONST ERROR_PREFIX      = 'PER1';
    protected $_storage;

    public function __construct(\Neuron\Person\Entity\Storage\Skeleton $storage, $data = array()) {

        $this->_storage = $storage;

        parent::__construct(
            $data, 
            array(
                new Map('id'),
                new Map('first_name'),
                new Map('last_name'),
                new Map('gender'),
                new Map('birth_date'),
                new Map('birth_place'),
                new Map('created_dtm'),
                new Map('created_by'),
                new Map('religion_id'),
                new Map('detail',null,true)
            ), 

            parent::CASE_INSENSITIVE
        );
    }

    public function religion(){
        $religion   = new Religion(Religion\Storage::factory($this->getDb(),$this->getConfig()));
        $religion->load($this->religion_id);
        return $religion;
    }
	
	public function load($id = null, $mode = Entity\Storage::LOADBY_ID,$params = array()) {
		if ($data = $this->_storage->load($id,$mode, $params)) {
            $this->push($data);
            return true;
		}
		return false;
	}

    public function saveEx($data = []){
        $result     = new Result;

        /* Validating */
        $filter = $this->validate($data);
        if($filter->code != 0){
            return $filter;
        }
        $data   = $filter->data;

        $update     = false;
        if(isset($data['id']) && !empty($data['id'])){
            $this->load($data['id']);
            unset($data['created_by']);
            $update = true;
        }else{
            $this->created_dtm = $this->_storage->now();
        }

        $param = $this->pull();
        foreach($data as $k => $v){
            if(array_key_exists($k, $param)){
                $param[$k] = $v;
            }
        }

        $this->push($param);
        
        /* Gagal save person */
        if(!$this->save($update)){
            $result->code = self::ERROR_PREFIX.'1';
            $result->info = "Failed saving person";
            return $result;
        }

        $result->data   = $this->pull();
        $result->info   = "Success saving person";
        return $result;
    }

    public function save($update = false){
		if ($id = $this->_storage->save($this, $update)) {
		  $this->id = $id;
		  return true;
		}
		return false;
    }

    public function delete($id) {
        return $this->_storage->remove($id);
    }

    public function validate($data){
        $result     = new Result;
        $isValid = true;
        $notEmpty = new NotEmpty();
        /* Validasi kolom yang wajib */

        /* Filter ['start']*/
            $filterAlnum        = new FilterAlnum(true);
            $filterDigits       = new FilterDigits();
        /* Filter ['end']*/

        /* [Validator ['start'] ] */
            /* Validator Name */
            $validatorName = new ValidatorChain();
            $validatorName->attach(new Alpha(true));
            $validatorName->attach(new NotEmpty());

            /* Validator Name */
            $validatorLastName = new ValidatorChain();
            $validatorLastName->attach(new Alpha(true));

            /* Validator Gender */
            $validatorGender = new ValidatorChain();
            $validatorGender->attach(new Alpha(false));
            $validatorGender->attach(new StringLength([
                'min' => 1,
                'max' => 1
            ]));

            $validatorGender->attach(new InArray(['haystack'  => ['F','M']]));
            
            /* Validator Birthday */
            $validatorBirthDay = new Date();

            /* Validator digits */
            $validatorDigits    = new ValidatorChain();
            $validatorDigits->attach(new Digits());
            $validatorDigits->attach(new NotEmpty());
        /* [Validator ['end'] ] */

        /* Validate input ['start'] */
            /* Validate first name */
            if(!$validatorName->isValid($data['first_name'])){
                $result->code = 1;
                $result->info = "First name column is not valid";
                return $result;
            }

            /* Validate last_name.. ga mandatory*/
            // if(!$validatorLastName->isValid($data['last_name'])){
            //     $result->code = 2;
            //     $result->info = "Last name column is not valid";
            //     return $result;
            // }

            /* Validate Gender */
            if(!$validatorGender->isValid($data['gender'])){
                $result->code = 3;
                $result->info = "Invalid gender";
                return $result;
            }
            /* Validate birth day */
            if(!$validatorBirthDay->isValid($data['birth_date'])){
                $result->code = 4;
                $result->info = "Invalid birth_date column";
                return $result;
            }
            /* Validate birthdate, pake name aja. */
            if(!$validatorName->isValid($data['birth_place'])){
                $result->code = 5;
                $result->info = "Invalid birth_place";
                return $result;
            }
        /* Validate input ['start'] */

        /* Filtering['start'] */
        $data['first_name']     = $filterAlnum->filter($data['first_name']);
        $data['last_name']      = $filterAlnum->filter($data['last_name']);
        $data['birth_place']    = $filterAlnum->filter($data['birth_place']);
        /* Filtering['end'] */

        $result->data = $data;
        return $result;

    }

    public function create($data = []){
        $result     = new Result;
        if(!isset($data['detail']) || empty($data['detail'])){
            $result->code = 1;
            $result->info = "Detail required!";
            return $result;
        }
        $dataDetail     = $data['detail'];
        unset($data['detail']);
        /* Create person */
        $savePerson = $this->saveEx($data);
        $dataPerson = $savePerson->data;
        if($savePerson->code != 0){
            return $savePerson;
        }

        // /* Save detail */
        // $detail         = new Detail(Detail\Storage::factory(
        //     $this->_storage->getAdapter(),
        //     $this->_storage->getConfig()
        // ));
        // $dataDetail['person_id'] = $this->id;
        // $saveDetail     = $detail->saveEx($dataDetail);
        // if($saveDetail->code != 0){
        //     $this->delete($this->id);
        //     return $saveDetail;
        // }
        // $this->detail = $detail;

        $result->data = $this;
        return $result;
    }

    /* @_@ ga standar, tapi kalo dihapus nyebabin error ke module lain.*/
    public function loadMap($id, $field= 'id'){
        $map = new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory($this->_storage->getAdapter(), $this->_storage->getConfig()));
        if($field == "user_id"){
            $map->load($id, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
        }else if($field == "person_id"){
            $map->load($id, \Neuron\Person\Map\Storage::LOADBY_PERSON_ID);
        }else{
            $map->load($id);
        }
        if($map->person_id){
            $this->load($map->person_id);
            $dataPerson = $this->pull();
            $dataPerson['ref_id'] = $dataPerson['id'];
        }
        $data = array_merge($map->pull(), $dataPerson);
        return $data;
    }
	
	public function countEmpGender(){
        $result         = new \Neuron\Generic\Result();
        $rows           = $this->_storage->countEmpGender();
        $result->data   = $rows;
        return $result;
    }
}
