<?php
namespace Neuron\Person\Religion\Storage;

use Neuron\Db\Storage\Pager;
use Neuron\Person\Religion;
use Neuron\Person\Religion\Storage;
use Zend\Db\Adapter\Adapter;
use Zend\Stdlib\ArrayUtils;
use Throwable;

class Mysql extends \Neuron\Db\Storage implements Skeleton
{
    public function load($id, $mode = Storage::LOADBY_ID)
    {
        try {
            $select = $this->select()
                ->from($this->_tables['person_religion']);

            switch ($mode) {
                case Storage::LOADBY_ID:
                    $select->where($this->__(['religion_id' => $id]));
                    break;
                default:
                    $select->where($this->__(['religion_id' => $id]));
            }

            return $this->fetchRow($select);
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function save(Religion $religion, $update = false)
    {
        try {
            /* mode update? */
            if ($update === true) {
                $update = $this->update()
                    ->table($this->_tables['person_religion'])
                    ->set($this->__($religion->pull()))
                    ->where($this->__(['religion_id' => $religion->religion_id]));
                if ($this->execute($update)) {
                    /* update sukses, return model id */
                    return $religion->religion_id;
                }
            } else {
                $insert = $this->insert()
                    ->into($this->_tables['person_religion'])
                    ->autoincrement($this->_('religion_id'))
                    ->values($this->__($religion->pull()));

                if ($id = $this->execute($insert)) {
                    /* insert sukses, return model id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = Storage::LIST_ALL, $params = null)
    {
        /* base select */
        $select = $this->select()
            ->from(['pr' => $this->_tables['person_religion']]);

        /* loadby page */
        if ($mode === Storage::LISTBY_PAGE) {
            /* get pager */
            $pager = Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('religion_id', 'a.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('name', 'a.'), '%' . trim($ids) . '%')
                    ->unnest();
            }
        } else {
            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                
            }
        }

        try {
            $rows = $this->fetchAll($select);
            $result = new \stdClass();
            $result->list = $rows;
            $result->pager = $pager;
            $total = isset($rows->total) ? $rows->total : 0;
            $result->countTotal = $total;
            return $result;
        } catch (\Throwable $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
}
